package co.ab180.airbridge.ue;

import android.app.Application;
import android.content.ContentProvider;
import android.content.ContentValues;
import android.database.Cursor;
import android.net.Uri;
import android.util.Log;

import androidx.annotation.NonNull;

import org.jetbrains.annotations.NotNull;

import java.lang.reflect.Method;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.Objects;

import co.ab180.airbridge.Airbridge;
import co.ab180.airbridge.AirbridgeConfig;
import co.ab180.airbridge.OnAttributionResultReceiveListener;

public class AirbridgeContentProvider extends ContentProvider {
    
    private static final String UNREAL_ENGINE_4_GAME_ACTIVITY = "com.epicgames.ue4.GameActivity";
    private static final String UNREAL_ENGINE_5_GAME_ACTIVITY = "com.epicgames.unreal.GameActivity";

    @Override
    public boolean onCreate() {
        try {
            Application app = (Application) Objects.requireNonNull(getContext()).getApplicationContext();

            AirbridgeConfig.Builder builder = new AirbridgeConfig.Builder(AirbridgeSettings.appName, AirbridgeSettings.appToken);
            builder.setPlatform("unreal");

            if (isNotEmpty(AirbridgeSettings.sdkSignatureSecretID) && isNotEmpty(AirbridgeSettings.sdkSignatureSecret)) {
                builder.setSDKSignatureSecret(AirbridgeSettings.sdkSignatureSecretID, AirbridgeSettings.sdkSignatureSecret);
            }

            if (isNotEmpty(AirbridgeSettings.customDomain)) {
                List<String> customDomains = new ArrayList<>();
                customDomains.add(AirbridgeSettings.customDomain);
                builder.setCustomDomains(customDomains);
            }

            builder
                    .setLogLevel(AirbridgeSettings.logLevel)
                    .setSessionTimeoutSeconds(AirbridgeSettings.sessionTimeoutSeconds)
                    .setUserInfoHashEnabled(AirbridgeSettings.userInfoHashEnabled)
                    .setLocationCollectionEnabled(AirbridgeSettings.locationCollectionEnabled)
                    .setTrackAirbridgeLinkOnly(AirbridgeSettings.trackAirbridgeLinkOnly)
                    .setAutoStartTrackingEnabled(AirbridgeSettings.autoStartTrackingEnabled);

            builder.setOnAttributionResultReceiveListener(new OnAttributionResultReceiveListener() {
                @Override
                public void onAttributionResultReceived(@NonNull Map<String, String> map) {
                    if (
                        processSetOnAttributionReceived(UNREAL_ENGINE_4_GAME_ACTIVITY, map) ||
                        processSetOnAttributionReceived(UNREAL_ENGINE_5_GAME_ACTIVITY, map)
                    ) { 
                        return; 
                    }
                    
                    Log.e("Airbridge Unreal Engine", "Couldn't handle {setOnAttributionResultReceiveListener}" +
                        " - failed to find GameActivity#airbridgeProcessSetOnAttributionReceived");
                }
            });
            
            Airbridge.init(app, builder.build());
        } catch (Throwable throwable) {
            Log.e("Airbridge Unreal Engine", "Couldn't initialize SDK", throwable);
        }
        return true;
    }

    @Override
    public Cursor query(@NotNull Uri uri, String[] projection, String selection, String[] selectionArgs, String sortOrder) {
        return null;
    }

    @Override
    public String getType(@NotNull Uri uri) {
        return null;
    }

    @Override
    public Uri insert(@NotNull Uri uri, ContentValues values) {
        return null;
    }

    @Override
    public int delete(@NotNull Uri uri, String selection, String[] selectionArgs) {
        return 0;
    }

    @Override
    public int update(@NotNull Uri uri, ContentValues values, String selection, String[] selectionArgs) {
        return 0;
    }
        
    private boolean processSetOnAttributionReceived(String className, @NonNull Map<String, String> map) {
        try {
            Class<?> clazz = Class.forName(className);
            Method method = clazz.getMethod("airbridgeProcessSetOnAttributionReceived", Map.class);
            method.invoke(null, map);
            return true;
        } catch (Throwable ignore) {
            return false;
        }
    }
    
    private boolean isNotEmpty(String string) {
        if (string == null) { return false; }
        return !(string.isEmpty());
    }
}
